unit MainDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, RCAPI, RCTYPE, CDTYPE, CDERRORCODES,
  CamDefines, Spin, OleCtrls, CamRemoteActiveXXControl1_TLB;

const
  PixelCountMax          = 32768;

type
     TRGBTriple = packed record
                    rgbtBlue  : byte;
                    rgbtGreen : byte;
                    rgbtRed   : byte;
                  end;
     pRGBTriple = ^TRGBTriple;
     TRGBTripleArray = array[0..PixelCountMax-1] of TRGBTriple;
     pRGBTripleArray = ^TRGBTripleArray;

//A record of supported RAW development parameters
type CamerRAWParaDevType =
           record
             WhiteBalanceSetting : array of RemoteFormatWBType;
           end;

var gWhiteBalanceSetArrayText : array[low(RemoteFormatWBType)..high(RemoteFormatWBType)] of string =
  ('-', 'Auto', 'Daylight', 'Cloudy', 'Tungsten', 'Fluorscent', 'Flash',
   'Fluorescent light', 'Custom', 'Custom1', 'Custom2', 'BW', 'Shade',
   'Kelvin', 'PCSet1', 'PCSet2', 'PCSet3');

type
  TFormMain = class(TForm)
    ButtonSrc: TButton;
    EditSource: TEdit;
    ButtonDst: TButton;
    EditDestination: TEdit;
    ButtonProcess: TButton;
    ProgressBar: TProgressBar;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    Label1: TLabel;
    EditKelvin: TEdit;
    LabelWB: TLabel;
    ButtonCreateRAW: TButton;
    ComboBoxWB: TComboBox;
    ButtonSetRawParam: TButton;
    ButtonGetRawParam: TButton;
    ButtonCloseRaw: TButton;
    ComboBoxTgtFormat: TComboBox;
    Label2: TLabel;
    CheckBoxDevelopJPEG: TCheckBox;
    CheckBoxDevelopThumb: TCheckBox;
    ButtonSelectICCProfile: TButton;
    EditICCProfile: TEdit;
    LabelJPEGQuality: TLabel;
    SpinEditJPEGQuality: TSpinEdit;
    CamRemoteActiveX: TCamRemoteActiveXX;    procedure ButtonSrcClick(Sender: TObject);
    procedure ButtonDstClick(Sender: TObject);
    procedure ButtonProcessClick(Sender: TObject);
    procedure ButtonCreateRAWClick(Sender: TObject);
    procedure ButtonSetRawParamClick(Sender: TObject);
    procedure ButtonGetRawParamClick(Sender: TObject);
    procedure ComboBoxWBChange(Sender: TObject);
    procedure ButtonCloseRawClick(Sender: TObject);
    procedure ButtonSelectICCProfileClick(Sender: TObject);
    procedure ComboBoxTgtFormatChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CamRemoteActiveXRawDevelopEvent(ASender: TObject;
      PercentageDone: Integer);
  private
    mCameraRAWParaDev  : CamerRAWParaDevType;  //Contains RAW development parameters possible to set
    mRAWObjectDevAvail : RAWObjectDevAvailType;
    procedure UpdateRemoteParamVCL;
    procedure SetRawDevParams(inRawDevParam : RAWObjectDevType);
    function  GetRawDevParams: RAWObjectDevType;
    procedure SetButtonsEnable(Active : boolean);
    function  Versiontext : string;
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}

procedure TFormMain.ButtonSrcClick(Sender: TObject);
begin
  if (OpenDialog.Execute) then
  begin
     EditSource.Text := OpenDialog.FileName;
  end;
end;

procedure TFormMain.ButtonDstClick(Sender: TObject);
begin
  if (SaveDialog.Execute) then
  begin
     EditDestination.Text := SaveDialog.FileName;
  end;
end;

procedure TFormMain.ButtonProcessClick(Sender: TObject);
var tgt_format       : TxBitsPerPixelType;
    file_format_type : TxFileFormatType;
begin
  tgt_format := BitsPerPixel48Bits;
  case ComboBoxTgtFormat.ItemIndex of
    0 : tgt_format := BitsPerPixel48Bits;
    1 : tgt_format := BitsPerPixel24Bits;
  end;
  if (CheckBoxDevelopJPEG.Checked) then
  begin
    CamRemoteActiveX.DevelopRAWPicture(ChangeFileExt(EditDestination.Text, '.jpg'),
                                       FileFormatJPEGInRaw,
                                       tgt_format,
                                       '',
                                       10);
  end;
  if (CheckBoxDevelopThumb.Checked) then
  begin
    CamRemoteActiveX.DevelopRAWPicture(ChangeFileExt(EditDestination.Text, '.thm.bmp'),
                                       FileFormatThumbnail,
                                       tgt_format,
                                       '',
                                       10);
  end;
  file_format_type := FileFormatTIFF;
  if (ComboBoxTgtFormat.ItemIndex = 2) then
  begin
    file_format_type := FileFormatJPEG;
  end;
  CamRemoteActiveX.DevelopRAWPicture(EditDestination.Text,
                                     file_format_type,
                                     tgt_format,
                                     EditICCProfile.Text,
                                     SpinEditJPEGQuality.Value);
end;

procedure TFormMain.UpdateRemoteParamVCL;
var i                           : integer;
    whitebalancesetting_counter : RemoteFormatWBType;
begin
  //Assign camera parameters possible to set
  //WhiteBalanceSetting
  i := 0;
  SetLength(mCameraRAWParaDev.WhiteBalanceSetting, 0);
  for whitebalancesetting_counter := Succ(Low(RemoteFormatWBType)) to High(RemoteFormatWBType) do
  begin
    if (mRAWObjectDevAvail.WhiteBalance[whitebalancesetting_counter]) then
    begin
       SetLength(mCameraRAWParaDev.WhiteBalanceSetting, i + 1);
       mCameraRAWParaDev.WhiteBalanceSetting[i] := whitebalancesetting_counter;
       i := i + 1;
    end;
  end;
  //WhiteBalanceSetting combobox
  ComboBoxWB.Enabled := false;
  ComboBoxWB.Items.Clear;
  for i := 0 to high(mCameraRAWParaDev.WhiteBalanceSetting) do begin
    ComboBoxWB.Enabled := true;
    ComboBoxWB.Items.Add(gWhiteBalanceSetArrayText[mCameraRAWParaDev.WhiteBalanceSetting[i]]);
  end;
end;

procedure TFormMain.ButtonCreateRAWClick(Sender: TObject);

  procedure set_remote_array(var remote_array : array of boolean; remote_string : string);
  var i : integer;
  begin
    for i := 0 to Length(remote_string) - 1 do
    begin
      remote_array[i] := false;
      if (remote_string[i + 1] = '1') then
      begin
        remote_array[i] := true;
      end;
    end;
  end;

begin
  CamRemoteActiveX.OpenRAWObject(EditSource.Text);
  set_remote_array(mRAWObjectDevAvail.WhiteBalance,
                   CamRemoteActiveX.OpenRAWObject_GetRemoteParamSupported(RAWParamWhiteBalanceSetting));
  SetButtonsEnable(true);
  UpdateRemoteParamVCL;
end;

//Sets new remote parameters in the VCLs in the parameters tab
procedure TFormMain.SetRawDevParams(inRawDevParam : RAWObjectDevType);
begin
  ComboBoxWB.ItemIndex := ComboBoxWB.Items.IndexOf(gWhiteBalanceSetArrayText[inRawDevParam.WhiteBalance]);
  EditKelvin.Text      := IntToStr(inRawDevParam.WhiteBalanceKelvin);
end;

//Gets remote parameters from the VCL in the remote tab
function TFormMain.GetRawDevParams: RAWObjectDevType;
begin
  if (ComboBoxWB.Enabled) then
    result.WhiteBalance  := mCameraRAWParaDev.WhiteBalanceSetting[ComboBoxWB.ItemIndex]
  else
    result.WhiteBalance  := RemoteFormatWBNotUsed;
  result.WhiteBalanceKelvin := StrToInt(EditKelvin.Text);
end;

procedure TFormMain.ButtonSetRawParamClick(Sender: TObject);
begin
  CamRemoteActiveX.SetRAWDevelopmentParameters(RAWParamWhiteBalanceSetting,
                                               Ord(GetRawDevParams.WhiteBalance));
  if (GetRawDevParams.WhiteBalance = RemoteFormatWBKelvin) then
  begin
    CamRemoteActiveX.SetRAWDevelopmentParameters(RAWParamWhiteBalanceKelvin,
                                                 GetRawDevParams.WhiteBalanceKelvin);
  end;
end;

procedure TFormMain.ButtonGetRawParamClick(Sender: TObject);
var raw_parameters : RAWObjectDevType;
begin
  FillChar(raw_parameters, sizeof(RAWObjectDevType), #0);
  raw_parameters.WhiteBalance :=
    RemoteFormatWBType(CamRemoteActiveX.GetRAWDevelopmentParameters(RAWParamWhiteBalanceSetting));
  raw_parameters.WhiteBalanceKelvin :=
    CamRemoteActiveX.GetRAWDevelopmentParameters(RAWParamWhiteBalanceKelvin);
  SetRawDevParams(raw_parameters);
end;


//TBD Dispose the RAW-object!!!
procedure TFormMain.ComboBoxWBChange(Sender: TObject);
begin
  EditKelvin.Enabled :=
    (ComboBoxWB.Text = gWhiteBalanceSetArrayText[RemoteFormatWBKelvin]);
end;

procedure TFormMain.ButtonCloseRawClick(Sender: TObject);
begin
  SetButtonsEnable(false);
  CamRemoteActiveX.CloseRAWObject;
end;

procedure TFormMain.SetButtonsEnable(Active : boolean);
begin
  ButtonCreateRAW.Enabled   := not Active;
  ButtonSetRawParam.Enabled := Active;
  ButtonGetRawParam.Enabled := Active;
  ButtonProcess.Enabled     := Active;
  ButtonCloseRaw.Enabled    := Active;
end;

procedure TFormMain.ButtonSelectICCProfileClick(Sender: TObject);
begin
  if (OpenDialog.Execute) then
  begin
     EditICCProfile.Text := OpenDialog.FileName;
  end;
end;

procedure TFormMain.ComboBoxTgtFormatChange(Sender: TObject);
begin
  SpinEditJPEGQuality.Enabled := ComboBoxTgtFormat.ItemIndex = 2;
end;

//Gets application version text
function TFormMain.Versiontext : string;
var
  size  : DWord;
  size2 : DWord;
  pt    : Pointer;
  pt2   : Pointer;
begin
  //Get the version of application
  result := '';
  size   := GetFileVersionInfoSize(PChar(ParamStr(0)), size2);
  if (size > 0) then
  begin
    GetMem(pt, size);
    try
       GetFileVersionInfo(PChar(ParamStr(0)), 0, size, pt);
       // show the fixed information
       VerQueryValue(pt, '\', pt2, size2);
       with TVSFixedFileInfo(pt2^) do
       begin
         result := result + IntToStr(HiWord(dwFileVersionMS)) + '.';
         result := result + IntToStr(LoWord(dwFileVersionMS)) + '.';
         result := result + IntToStr(HiWord(dwFileVersionLS));
         result := result + ' Build:' + IntToStr(LoWord(dwFileVersionLS));
       end;
    finally
      FreeMem(pt);
    end;
  end;
end;


procedure TFormMain.FormCreate(Sender: TObject);
begin
  FormMain.Caption := FormMain.Caption + ' ' + Versiontext;
end;

procedure TFormMain.CamRemoteActiveXRawDevelopEvent(ASender: TObject;
  PercentageDone: Integer);
begin
   ProgressBar.Position := PercentageDone;
end;

end.
